/*
 * Copyright 2010 Jerome Glisse <glisse@freedesktop.org>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * on the rights to use, copy, modify, merge, publish, distribute, sub
 * license, and/or sell copies of the Software, and to permit persons to whom
 * the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHOR(S) AND/OR THEIR SUPPLIERS BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
 * OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
 * USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Authors:
 *      Jerome Glisse
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include "radeon_pci.h"
#include "atom.h"

struct ctx {
    struct radeon_pci       pradeon;
    void                    *bios;
    unsigned                bios_size;
    struct atom_context     *atom_ctx;
};

/* ATOM accessor methods */
static uint32_t cail_pll_read(struct card_info *info, uint32_t reg)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);\
    return 0;
}

static void cail_pll_write(struct card_info *info, uint32_t reg, uint32_t val)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);\
}

static uint32_t cail_mc_read(struct card_info *info, uint32_t reg)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);\
    return 0;
}

static void cail_mc_write(struct card_info *info, uint32_t reg, uint32_t val)
{
    fprintf(stderr, "%s:%d unsupported operation\n", __func__, __LINE__);
}

static void cail_reg_write(struct card_info *info, uint32_t reg, uint32_t val)
{
    struct ctx *ctx = info->ctx;

    radeon_mmio_wr32(&ctx->pradeon, reg << 2, val);
}

static uint32_t cail_reg_read(struct card_info *info, uint32_t reg)
{
    struct ctx *ctx = info->ctx;

    return radeon_mmio_rd32(&ctx->pradeon, reg << 2);
}

#define R600_BIOS_0_SCRATCH               0x1724
#define R600_BIOS_1_SCRATCH               0x1728
#define R600_BIOS_2_SCRATCH               0x172c
#define R600_BIOS_3_SCRATCH               0x1730
#define R600_BIOS_4_SCRATCH               0x1734
#define R600_BIOS_5_SCRATCH               0x1738
#define R600_BIOS_6_SCRATCH               0x173c
#define R600_BIOS_7_SCRATCH               0x1740

void atom_initialize_bios_scratch_regs(struct ctx *ctx)
{
    uint32_t bios_2_scratch, bios_6_scratch;

    bios_2_scratch = radeon_mmio_rd32(&ctx->pradeon, R600_BIOS_2_SCRATCH);
    bios_6_scratch = radeon_mmio_rd32(&ctx->pradeon, R600_BIOS_6_SCRATCH);

    /* let the bios control the backlight */
    bios_2_scratch &= ~ATOM_S2_VRI_BRIGHT_ENABLE;

    /* tell the bios not to handle mode switching */
    bios_6_scratch |= ATOM_S6_ACC_BLOCK_DISPLAY_SWITCH;
    bios_6_scratch &= ~ATOM_S6_ACC_BLOCK_DISPLAY_SWITCH;

    radeon_mmio_wr32(&ctx->pradeon, R600_BIOS_2_SCRATCH, bios_2_scratch);
    radeon_mmio_wr32(&ctx->pradeon, R600_BIOS_6_SCRATCH, bios_6_scratch);
}

int atombios_init(struct ctx *ctx)
{
    struct card_info atom_card_info = {
        .reg_read       = cail_reg_read,
        .reg_write      = cail_reg_write,
        .ioreg_read     = cail_reg_read,
        .ioreg_write    = cail_reg_write,
        .mc_read        = cail_mc_read,
        .mc_write       = cail_mc_write,
        .pll_read       = cail_pll_read,
        .pll_write      = cail_pll_write,
    };

    atom_card_info.ctx = ctx;
    ctx->atom_ctx = atom_parse(&atom_card_info, ctx->bios);
    if (ctx->atom_ctx == NULL) {
        return -EINVAL;
    }
    atom_initialize_bios_scratch_regs(ctx);
    atom_allocate_fb_scratch(ctx->atom_ctx);
    return 0;
}

void atombios_fini(struct ctx *ctx)
{
    if (ctx->atom_ctx) {
        free(ctx->atom_ctx->scratch);
        free(ctx->atom_ctx);
    }
}

struct log {
    unsigned data[8];
};

static struct log logs[] = {
// 1024x768
};


static void process_table(struct ctx *ctx,
                          unsigned index,
                          unsigned dw0,
                          unsigned dw1,
                          unsigned dw2,
                          unsigned dw3,
                          unsigned dw4,
                          unsigned dw5,
                          unsigned dw6)
{
    struct atom_context *atom_ctx = ctx->atom_ctx;
    uint32_t args[8];

    args[0] = dw0;
    args[1] = dw1;
    args[2] = dw2;
    args[3] = dw3;
    args[4] = dw4;
    args[5] = dw5;
    args[6] = dw6;
    atom_execute_table(atom_ctx, index, args);
}

static const char *aux_status[] = {
    "AUX_NATIVE_REPLY_ACK",
    "AUX_NATIVE_REPLY_NACK",
    "AUX_NATIVE_REPLY_DEFER",
    "AUX_NATIVE_REPLY_MASK",
};

static void process_aux(struct ctx *ctx,
                        unsigned channel_id, 
                        unsigned hpd_id,
                        unsigned delay,
                        uint8_t *send,
                        uint8_t *recv)
{
    struct atom_context *atom_ctx = ctx->atom_ctx;
    PROCESS_AUX_CHANNEL_TRANSACTION_PARAMETERS_V2 args;
    int index = GetIndexIntoMasterTable(COMMAND, ProcessAuxChannelTransaction);
    unsigned char *base;
    unsigned ack, i, reg;

    memset(&args, 0, sizeof(args));
    base = (unsigned char *)(atom_ctx->scratch + 1);
    memcpy(base, send, 20);

    reg = send[0] | (send[1] << 8);

    args.lpAuxRequest = 0 + 4;
    args.lpDataOut = 16 + 4;
    args.ucDataOutLen = 0;
    args.ucChannelID = channel_id;
    args.ucDelay = delay;
    args.ucHPD_ID = hpd_id;

#define AUX_NATIVE_REPLY_ACK    (0x0 << 4)
#define AUX_NATIVE_REPLY_NACK   (0x1 << 4)
#define AUX_NATIVE_REPLY_DEFER  (0x2 << 4)
#define AUX_NATIVE_REPLY_MASK   (0x3 << 4)

    for (i = 0; i < 4; i++) {
        atom_execute_table(atom_ctx, index, (uint32_t *)&args);
        ack = args.ucReplyStatus & AUX_NATIVE_REPLY_MASK;
        switch (args.ucReplyStatus) {
        case 1:
            fprintf(stderr, "%s:%d aux timeout\n", __func__, __LINE__);
            goto finish;
        case 2:
            continue;
        default:
            fprintf(stderr, "%s:%d aux error (0x%02x)\n", __func__, __LINE__, args.ucReplyStatus);
            goto finish;
        }
        switch (ack) {
        case AUX_NATIVE_REPLY_ACK:
            goto finish;
        case AUX_NATIVE_REPLY_NACK:
        case AUX_NATIVE_REPLY_DEFER:
        default:
            usleep(100);
            break;
        }
    }

finish:

    fprintf(stderr, "%s:%d status (0x%02x) %s [%d tries]\n", __func__, __LINE__,
            args.ucReplyStatus, aux_status[ack >> 4], i);

    if (!args.ucReplyStatus) {
        if (args.ucDataOutLen) {
            memcpy(recv, base + 16, args.ucDataOutLen);
            fprintf(stderr, "%s:%d 0x%04x recv = {", __func__, __LINE__, reg);
            for (i = 0; i <= args.ucDataOutLen; i++) {
                fprintf(stderr, " 0x%02x,", recv[i]);
            }
            fprintf(stderr, "};\n");
        }
    }
}

#define RDIS 0

void replay_f(struct ctx *ctx)
{
    uint8_t send[64];
    uint8_t recv[64];

    /* vbios */
#if 0
    process_table(ctx, 0, 0x00004e20, 0x0001048c, 0x0d200000, 0x1ddd0000, 0x00002816, 0x6430024b, 0x00000000);
#endif
#if 0
    process_table(ctx, 76, 0x07843103, 0x004c2a33, 0x16770008, 0x64300257, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 76, 0x07803105, 0x004c2a00, 0x16770001, 0x64300257, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 50, 0x07003105, 0x0000004c, 0x16810001, 0x64300257, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 0
    process_table(ctx, 9, 0x00000002, 0x00010200, 0x000c1e12, 0x00012d78, 0xb1f8220c, 0x00010ff8, 0x01080e4a);
#endif
#if 0
    send[ 0] = 0x00;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x47;
    send[ 4] = 0x02;    send[ 5] = 0x1b;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x00;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x41;
    send[ 4] = 0x02;    send[ 5] = 0x1b;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
#endif
#if 0
    process_table(ctx, 50, 0x14003105, 0x00000200, 0x22240001, 0x0ff8b1f8, 0x0e4a0001, 0x00030108, 0x00030da0);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 54, 0x00322618, 0x80002618, 0x228d90a0, 0x0ff8b1f8, 0x0e4a0008, 0x00030108, 0x00030da0);
    process_table(ctx, 54, 0x00322618, 0x80002618, 0x229490a2, 0x0ff8b1f8, 0x0e4a0008, 0x00030108, 0x00030da0);
#endif
#if 0
    process_table(ctx, 4, 0x0c0003d4, 0x0000ff03, 0x2ddd0001, 0x16df000c, 0x0dab168e, 0x5b8a69a7, 0x6b6d5aba);
#endif
#if 1
    process_table(ctx, 34, 0x17dd0100, 0x16b41886, 0x0dab1699, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25, 0x64300267);
#endif
#if 0
    process_table(ctx, 6, 0x17dd0000, 0x16be1886, 0x16990201, 0x69a70dab, 0x5aba5b8a, 0x6b256b6d, 0x0267c000);
#endif
#if 0
    process_table(ctx, 44, 0x17dd0100, 0x18941886, 0x69a70db0, 0x5aba5b8a, 0x6b256b6d, 0x0267c000, 0x00006430);
#endif
#if 0
    process_table(ctx, 76, 0x00842221, 0x15170400, 0x135d0008, 0x0db3125c, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25);
#endif
#if 1
    process_table(ctx, 50, 0x00003105, 0x0000004c, 0x134b0001, 0x0db3125c, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25);
#endif
#if 1
    process_table(ctx, 76, 0x00800000, 0x15170400, 0x135d0001, 0x0db3125c, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25);
#endif
#if 1
    process_table(ctx, 35, 0xffff0000, 0x0db31272, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25, 0x64300267, 0x00000000);
#endif
#if 0
    process_table(ctx, 35, 0x00000001, 0x0db31272, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25, 0x64300267, 0x00000000);
#endif
#if 0
    process_table(ctx, 12, 0x000000ff, 0x00000000, 0x00000001, 0x00000123, 0x12a917e5, 0x128c0000, 0x69a70db3);
    process_table(ctx, 43, 0x05000300, 0x00010000, 0x00010001, 0x1f250000, 0x060402ff, 0x69a70db9, 0x5aba5b8a);
    process_table(ctx, 43, 0x05000300, 0x00000001, 0x00000000, 0x1f2d0100, 0x060402ff, 0x69a70db9, 0x5aba5b8a);
    process_table(ctx, 12, 0xe10000ff, 0x00000000, 0x00ff0002, 0x00000000, 0xe10034c0, 0x203d0000, 0x69a70dbc);
#endif
#if 0
    process_table(ctx, 17, 0x00216978, 0x22002210, 0x204e0001, 0x69a70dbc, 0x5aba5b8a, 0x6b256b6d, 0x0267c000);
    process_table(ctx, 12, 0x0fa00000, 0x0101000d, 0x10002100, 0x0007a120, 0x0fa034c0, 0x205d0001, 0x69a70dbc);
    process_table(ctx, 49, 0x01000320, 0x001c0258, 0x00800028, 0x00040001, 0x00000000, 0xb6901b00, 0x207c0001);
    process_table(ctx, 43, 0x05000300, 0x00010100, 0x00010001, 0x1f250001, 0x00000102, 0x69a70dc2, 0x5aba5b8a);
    process_table(ctx, 43, 0x05000300, 0x00000101, 0x00000101, 0x1f2d0101, 0x00000102, 0x69a70dc2, 0x5aba5b8a);
    process_table(ctx, 37, 0x0df90100, 0x69a70dca, 0x5aba5b8a, 0x6b256b6d, 0x0267c000, 0x00006430, 0x00000000);
#endif
#if 0
    process_table(ctx, 45, 0x00250100, 0x0dd5a420, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25, 0x64300267, 0x00000000);
    process_table(ctx, 45, 0x00250101, 0x0dd5a428, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25, 0x64300267, 0x00000000);
#endif
#if 0
    process_table(ctx, 33, 0x20b40200, 0x208e0000, 0x00000000, 0x5ba60ddc, 0x6b6d5aba, 0xc0006b25, 0x64300267);
    process_table(ctx, 33, 0x17d50001, 0x20960000, 0x00000000, 0x5ba60ddc, 0x6b6d5aba, 0xc0006b25, 0x64300267);
    process_table(ctx, 35, 0x17dd0100, 0x0de41886, 0x5aba5ba6, 0x6b256b6d, 0x0267c000, 0x00006430, 0x00000000);
    process_table(ctx, 44, 0x17dd0000, 0x18941886, 0x5ba60de9, 0x6b6d5aba, 0xc0006b25, 0x64300267, 0x00000000);
    process_table(ctx, 42, 0x13000300, 0x12c60001, 0x0dec122e, 0x5aba5ba6, 0x6b256b6d, 0x0267c000, 0x00006430);
    process_table(ctx, 4, 0x0f010fa0, 0x02000100, 0x12db0001, 0x0dec122e, 0x5aba5ba6, 0x6b256b6d, 0x0267c000);
    process_table(ctx, 4, 0x10010fa0, 0x00000011, 0x12e80001, 0x0dec122e, 0x5aba5ba6, 0x6b256b6d, 0x0267c000);
    process_table(ctx, 50, 0x0f010fa0, 0x02000100, 0x13130001, 0x122e12eb, 0x5ba60dec, 0x6b6d5aba, 0xc0006b25);
    process_table(ctx, 76, 0x01806978, 0x00011401, 0x13200001, 0x122e12eb, 0x5ba60dec, 0x6b6d5aba, 0xc0006b25);
    process_table(ctx, 4, 0x08006978, 0x0000ff03, 0x2ddd0001, 0x2f858008, 0x12eb1323, 0x0dec122e, 0x5aba5ba6);
    send[ 0] = 0x00;    send[ 1] = 0x06;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0x80;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x01;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x81;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x00;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x0a;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x07;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x09003e00, 0x0000ff03, 0x2ddd0001, 0x2fe70009, 0x12eb1323, 0x0dec122e, 0x5aba5ba6);
    process_table(ctx, 76, 0x0b800000, 0x00010000, 0x320e0001, 0x13232ffd, 0x122e12eb, 0x5ba60dec, 0x6b6d5aba);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0xca;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0a000000, 0x0000ff03, 0x2ddd0001, 0x3081010a, 0x12eb1323, 0x0dec122e, 0x5aba5ba6);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x02;    send[ 5] = 0x01;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x07;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0b003e00, 0x0000ff03, 0x2ddd0001, 0x30f7000b, 0x12eb1323, 0x0dec122e, 0x5aba5ba6);
    process_table(ctx, 50, 0x01010fa0, 0x02000100, 0x132d0001, 0x122e12eb, 0x5ba60dec, 0x6b6d5aba, 0xc0006b25);
    process_table(ctx, 50, 0x10003105, 0x9fff0032, 0x13350001, 0x122e12eb, 0x5ba60dec, 0x6b6d5aba, 0xc0006b25);
    process_table(ctx, 6, 0x17dd0100, 0x16be1886, 0x16a30000, 0x5ba60df1, 0x6b6d5aba, 0xc0006b25, 0x64300267);
#endif
#if 0
    process_table(ctx, 34, 0x17dd0000, 0x16ca1886, 0x0df116a3, 0x5aba5ba6, 0x6b256b6d, 0x0267c000, 0x00006430);
    process_table(ctx, 4, 0x0d0003c5, 0x0000ff03, 0x2ddd0001, 0x16e4000d, 0x0df116a8, 0x5aba5ba6, 0x6b256b6d);
    process_table(ctx, 58, 0x904e0000, 0xa3830008, 0xc0006b33, 0x6430026a, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 9, 0x00000002, 0x00010200, 0x000c1e12, 0x00012d78, 0xb1f8220c, 0x00010ff8, 0x01080e4a);
    send[ 0] = 0x00;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x47;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x00;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x41;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 50, 0x14003105, 0x00000200, 0x22240001, 0x0ff8b1f8, 0x0e4a0001, 0x00030108, 0x00030da0);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x20;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x30;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x40;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x50;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x60;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x40;    send[ 3] = 0x50;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x50;    send[ 3] = 0x4f;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x50;    send[ 1] = 0x00;    send[ 2] = 0x10;    send[ 3] = 0x30;
    send[ 4] = 0x70;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 54, 0x00322618, 0x80002618, 0x228d90a0, 0x0ff8b1f8, 0x0e4a0008, 0x00030108, 0x00030da0);
    process_table(ctx, 54, 0x00322618, 0x80002618, 0x229490a2, 0x0ff8b1f8, 0x0e4a0008, 0x00030108, 0x00030da0);
#endif
#if 0
    process_table(ctx, 4, 0x0c0003d4, 0x0000ff03, 0x2ddd0001, 0x16df000c, 0x0dab168e, 0x5b8a69a7, 0x00005a99);
    process_table(ctx, 34, 0x17dd0100, 0x16b41886, 0x0dab1699, 0x5b8a69a7, 0x00005a99, 0x00000000, 0x00000000);
    process_table(ctx, 6, 0x17dd0000, 0x16be1886, 0x16990201, 0x69a70dab, 0x5a995b8a, 0x00000000, 0x00000000);
    process_table(ctx, 44, 0x17dd0100, 0x18941886, 0x69a70db0, 0x5a995b8a, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 76, 0x00842221, 0x15170400, 0x135d0008, 0x0db3125c, 0x5b8a69a7, 0x00005a99, 0x00000000);
#endif
#if 0
    process_table(ctx, 17, 0x00216978, 0x22002210, 0x349a0001, 0x0db31257, 0x5b8a69a7, 0x00005a99, 0x00000000);
#endif
#if 1
    process_table(ctx, 36, 0x0fa00000, 0x0101000d, 0x10002100, 0x0007a120, 0x0fa034c0, 0x34a90001, 0x0db31257);
    process_table(ctx, 35, 0x17e50001, 0x0db31272, 0x5b8a69a7, 0x00005a99, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 0
    process_table(ctx, 12, 0x000000ff, 0x01010000, 0x00000001, 0x00000123, 0x12a917e5, 0x128c0000, 0x69a70db3);
#endif
#if 0
    process_table(ctx, 43, 0x05000300, 0x00010000, 0x00010001, 0x1f250000, 0x060402ff, 0x69a70db9, 0x5a995b8a);
    process_table(ctx, 43, 0x05000300, 0x00000001, 0x00000101, 0x1f2d0100, 0x060402ff, 0x69a70db9, 0x5a995b8a);
#endif
#if 0
    process_table(ctx, 12, 0xe10000ff, 0x00000000, 0x00ff0002, 0x00000000, 0xe10034c0, 0x203d0000, 0x69a70dbc);
#endif
#if 1
    process_table(ctx, 17, 0x00216978, 0x22002210, 0x204e0001, 0x69a70dbc, 0x5a995b8a, 0x00000000, 0x00000000);
    process_table(ctx, 12, 0x0fa00000, 0x0101000d, 0x10002100, 0x0007a120, 0x0fa034c0, 0x205d0001, 0x69a70dbc);
    process_table(ctx, 49, 0x01000320, 0x001c0258, 0x00800028, 0x00040001, 0x00000000, 0xb6901b00, 0x207c0001);
#endif
#if 0
    process_table(ctx, 43, 0x05000300, 0x00010100, 0x00010001, 0x1f250001, 0x00000102, 0x69a70dc2, 0x5a995b8a);
    process_table(ctx, 43, 0x05000300, 0x00000101, 0x00000101, 0x1f2d0101, 0x00000102, 0x69a70dc2, 0x5a995b8a);
#endif
#if 0
    process_table(ctx, 37, 0x0df90100, 0x69a70dca, 0x5a995b8a, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 0
    process_table(ctx, 45, 0x00250100, 0x0dd5a420, 0x5b8a69a7, 0x00005a99, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 45, 0x00250101, 0x0dd5a428, 0x5b8a69a7, 0x00005a99, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 0
    process_table(ctx, 33, 0x20b40200, 0x208e0000, 0x00000000, 0x5ba60ddc, 0x00005a99, 0x00000000, 0x00000000);
    process_table(ctx, 33, 0x17d50001, 0x20960000, 0x00000000, 0x5ba60ddc, 0x00005a99, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 35, 0x17dd0100, 0x0de41886, 0x5a995ba6, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 0
    process_table(ctx, 44, 0x17dd0000, 0x18941886, 0x5ba60de9, 0x00005a99, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 42, 0x13000300, 0x12c60001, 0x0dec122e, 0x5a995ba6, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x0f010fa0, 0x02000100, 0x12db0001, 0x0dec122e, 0x5a995ba6, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x10010fa0, 0x00000011, 0x12e80001, 0x0dec122e, 0x5a995ba6, 0x00000000, 0x00000000);
    process_table(ctx, 50, 0x0f010fa0, 0x02000100, 0x13130001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
    process_table(ctx, 76, 0x01806978, 0x00011401, 0x13200001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
    process_table(ctx, 4, 0x08006978, 0x0000ff00, 0x2ddd0001, 0x2f858008, 0x12eb1323, 0x0dec122e, 0x5a995ba6);
#endif
#if 1
    send[ 0] = 0x00;    send[ 1] = 0x06;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0x80;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x01;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x81;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x00;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x0a;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x07;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x09003e00, 0x0000ff00, 0x2ddd0001, 0x2fe70009, 0x12eb1323, 0x0dec122e, 0x5a995ba6);
    process_table(ctx, 76, 0x0b800000, 0x00010000, 0x320e0001, 0x13232ffd, 0x122e12eb, 0x5ba60dec, 0x00005a99);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0xd0;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
#endif
#if 0
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
#endif
#if 1
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0a000000, 0x0000ff00, 0x2ddd0001, 0x3081010a, 0x12eb1323, 0x0dec122e, 0x5a995ba6);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x02;    send[ 5] = 0x01;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
#endif
#if 0
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
#endif
#if 1
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x07;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0b003e00, 0x0000ff00, 0x2ddd0001, 0x30f7000b, 0x12eb1323, 0x0dec122e, 0x5a995ba6);
    process_table(ctx, 50, 0x01010fa0, 0x02000100, 0x132d0001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
    process_table(ctx, 50, 0x10003105, 0x9fff0032, 0x13350001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
#endif
#if 0
    process_table(ctx, 6, 0x17dd0100, 0x16be1886, 0x16a30000, 0x5ba60df1, 0x00005a99, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 34, 0x17dd0000, 0x16ca1886, 0x0df116a3, 0x5a995ba6, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 4, 0x0d0003c5, 0x0000ff00, 0x2ddd0001, 0x16e4000d, 0x0df116a8, 0x5a995ba6, 0x00000000);
#endif
}



void replay_f2(struct ctx *ctx)
{
    uint8_t send[64];
    uint8_t recv[64];

    /* vbios */
#if 1
    process_table(ctx, 76, 0x07803105, 0x004c2a00, 0x16770001, 0x64300257, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 50, 0x07003105, 0x0000004c, 0x16810001, 0x64300257, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 34, 0x17dd0100, 0x16b41886, 0x0dab1699, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25, 0x64300267);
#endif
#if 1
    process_table(ctx, 50, 0x00003105, 0x0000004c, 0x134b0001, 0x0db3125c, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25);
#endif
#if 1
    process_table(ctx, 76, 0x00800000, 0x15170400, 0x135d0001, 0x0db3125c, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25);
#endif
#if 1
    process_table(ctx, 35, 0xffff0000, 0x0db31272, 0x5b8a69a7, 0x6b6d5aba, 0xc0006b25, 0x64300267, 0x00000000);
#endif
#if 1
    process_table(ctx, 36, 0x0fa00000, 0x0101000d, 0x10002100, 0x0007a120, 0x0fa034c0, 0x34a90001, 0x0db31257);
    process_table(ctx, 35, 0x17e50001, 0x0db31272, 0x5b8a69a7, 0x00005a99, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 17, 0x00216978, 0x22002210, 0x204e0001, 0x69a70dbc, 0x5a995b8a, 0x00000000, 0x00000000);
    process_table(ctx, 12, 0x0fa00000, 0x0101000d, 0x10002100, 0x0007a120, 0x0fa034c0, 0x205d0001, 0x69a70dbc);
    process_table(ctx, 49, 0x01000320, 0x001c0258, 0x00800028, 0x00040001, 0x00000000, 0xb6901b00, 0x207c0001);
#endif
#if 1
    process_table(ctx, 35, 0x17dd0100, 0x0de41886, 0x5a995ba6, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 42, 0x13000300, 0x12c60001, 0x0dec122e, 0x5a995ba6, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x0f010fa0, 0x02000100, 0x12db0001, 0x0dec122e, 0x5a995ba6, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x10010fa0, 0x00000011, 0x12e80001, 0x0dec122e, 0x5a995ba6, 0x00000000, 0x00000000);
    process_table(ctx, 50, 0x0f010fa0, 0x02000100, 0x13130001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
    process_table(ctx, 76, 0x01806978, 0x00011401, 0x13200001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
    process_table(ctx, 4, 0x08006978, 0x0000ff00, 0x2ddd0001, 0x2f858008, 0x12eb1323, 0x0dec122e, 0x5a995ba6);
#endif


    // kms training
process_table(ctx, 76, 0x01826978, 0x00000002, 0x00000000, 0x28080301, 0, 0, 0);
send[ 0] = 0x0e; send[ 1] = 0x00; send[ 2] = 0x90; send[ 3] = 0x40;
send[ 4] = 0x00; send[ 5] = 0xe8; send[ 6] = 0xb2; send[ 7] = 0x2f;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x02; send[ 1] = 0x00; send[ 2] = 0x90; send[ 3] = 0x40;
send[ 4] = 0x00; send[ 5] = 0xe8; send[ 6] = 0xb2; send[ 7] = 0x2f;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x00; send[ 1] = 0x06; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x01; send[ 5] = 0x00; send[ 6] = 0x00; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x07; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x10; send[ 5] = 0x00; send[ 6] = 0x00; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x01; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x82; send[ 5] = 0x00; send[ 6] = 0x00; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x00; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x0a; send[ 5] = 0x00; send[ 6] = 0x00; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 0, send, recv);
process_table(ctx, 4, 0x08013a02, 0x00020200, 0xce000201, 0x8801ff59, 0, 0, 0);
send[ 0] = 0x02; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x00; send[ 5] = 0xce; send[ 6] = 0x59; send[ 7] = 0xff;
process_aux(ctx, 145, 1, 0, send, recv);
process_table(ctx, 4, 0x09013a02, 0x00020200, 0x00010201, 0x1eb00000, 0, 0, 0);
send[ 0] = 0x02; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x01; send[ 5] = 0xce; send[ 6] = 0x59; send[ 7] = 0xff;
process_aux(ctx, 145, 1, 0, send, recv);
process_table(ctx, 76, 0x0b820000, 0x00000002, 0x00000000, 0xce000301, 0, 0, 0);
send[ 0] = 0x03; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x61;
send[ 4] = 0x00; send[ 5] = 0x00; send[ 6] = 0x00; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x02; send[ 1] = 0x02; send[ 2] = 0x90; send[ 3] = 0x45;
send[ 4] = 0x00; send[ 5] = 0xd1; send[ 6] = 0x6c; send[ 7] = 0x04;
process_aux(ctx, 145, 1, 10, send, recv);
process_table(ctx, 4, 0x0a013a02, 0x00020200, 0x00000201, 0x00000000, 0, 0, 0);
send[ 0] = 0x02; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x02; send[ 5] = 0xce; send[ 6] = 0x59; send[ 7] = 0xff;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x02; send[ 1] = 0x02; send[ 2] = 0x90; send[ 3] = 0x45;
send[ 4] = 0xff; send[ 5] = 0x02; send[ 6] = 0x00; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 10, send, recv);
process_table(ctx, 76, 0x0b820800, 0x00000002, 0x00000000, 0x00440301, 0, 0, 0);
send[ 0] = 0x03; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x61;
send[ 4] = 0x08; send[ 5] = 0x08; send[ 6] = 0x00; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 0, send, recv);
send[ 0] = 0x02; send[ 1] = 0x02; send[ 2] = 0x90; send[ 3] = 0x45;
send[ 4] = 0x00; send[ 5] = 0x28; send[ 6] = 0xb3; send[ 7] = 0x00;
process_aux(ctx, 145, 1, 10, send, recv);
send[ 0] = 0x02; send[ 1] = 0x01; send[ 2] = 0x80; send[ 3] = 0x50;
send[ 4] = 0x00; send[ 5] = 0xac; send[ 6] = 0x85; send[ 7] = 0xda;
process_aux(ctx, 145, 1, 0, send, recv);
process_table(ctx, 4, 0x0b013a02, 0x00020200, 0x00000201, 0x00000000, 0, 0, 0);
        // KMS training end



#if 1
    process_table(ctx, 4, 0x0b003e00, 0x0000ff00, 0x2ddd0001, 0x30f7000b, 0x12eb1323, 0x0dec122e, 0x5a995ba6);
    process_table(ctx, 50, 0x01010fa0, 0x02000100, 0x132d0001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
    process_table(ctx, 50, 0x10003105, 0x9fff0032, 0x13350001, 0x122e12eb, 0x5ba60dec, 0x00005a99, 0x00000000);
#endif
#if 1
    process_table(ctx, 34, 0x17dd0000, 0x16ca1886, 0x0df116a3, 0x5a995ba6, 0x00000000, 0x00000000, 0x00000000);
#endif
#if 1
    process_table(ctx, 4, 0x0d0003c5, 0x0000ff00, 0x2ddd0001, 0x16e4000d, 0x0df116a8, 0x5a995ba6, 0x00000000);
#endif
}

void replay_f3(struct ctx *ctx)
{
    uint8_t send[64];
    uint8_t recv[64];

    process_table(ctx, 34, 0x17dd0100, 0x16b41886, 0x1ecb1699, 0x4f3f5317, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 6, 0x17dd0000, 0x16be1886, 0x16990201, 0x53171ecb, 0x00004f3f, 0x00000000, 0x00000000);
    process_table(ctx, 76, 0x00842221, 0x15170000, 0x135d0008, 0x1ed3125c, 0x4f3f5317, 0x00000000, 0x00000000);
    process_table(ctx, 17, 0x00216978, 0x22002210, 0x349a0001, 0x1ed31257, 0x4f3f5317, 0x00000000, 0x00000000);
    process_table(ctx, 36, 0x3f480000, 0x0101000d, 0x10002100, 0x0007a120, 0x3f4834c0, 0x34a90001, 0x1ed31257);
    process_table(ctx, 35, 0x17e50001, 0x1ed31272, 0x4f3f5317, 0x00000000, 0x00000000, 0x00000000, 0x00000176);
    process_table(ctx, 17, 0x00216978, 0x22002210, 0x204e0001, 0x53171edc, 0x00004f3f, 0x00000000, 0x00000000);
    process_table(ctx, 12, 0x3f480000, 0x0101000d, 0x10002100, 0x0007a120, 0x3f4834c0, 0x205d0001, 0x53171edc);
    process_table(ctx, 49, 0x02300640, 0x003204b0, 0x00c00040, 0x00030001, 0x00000000, 0xb6901b00, 0x207c0001);
    process_table(ctx, 35, 0x17dd0100, 0x1efd1886, 0x4f3f5317, 0x00000000, 0x00000000, 0x00000000, 0x00000176);
    process_table(ctx, 42, 0x13000300, 0x12c60001, 0x1f05122e, 0x4f3f5317, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x0f013f48, 0x02000200, 0x12db0001, 0x1f05122e, 0x4f3f5317, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x10013f48, 0x00000011, 0x12e80001, 0x1f05122e, 0x4f3f5317, 0x00000000, 0x00000000);
    process_table(ctx, 50, 0x0f013f48, 0x02000200, 0x13130001, 0x122e12eb, 0x53171f05, 0x00004f3f, 0x00000000);
    process_table(ctx, 76, 0x01806978, 0x00011402, 0x13200001, 0x122e12eb, 0x53171f05, 0x00004f3f, 0x00000000);
    process_table(ctx, 4, 0x08006978, 0x0000ff03, 0x2ddd0001, 0x2f858008, 0x12eb1323, 0x1f05122e, 0x4f3f5317);
    send[ 0] = 0x00;    send[ 1] = 0x06;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0x80;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x01;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x82;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x00;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x0a;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x07;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x09003e00, 0x0000ff03, 0x2ddd0001, 0x2fe70009, 0x12eb1323, 0x1f05122e, 0x4f3f5317);
    process_table(ctx, 76, 0x0b800000, 0x00010000, 0x320e0001, 0x13232ffd, 0x122e12eb, 0x53171f05, 0x00004f3f);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0x2e;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0a000000, 0x0000ff03, 0x2ddd0001, 0x3081010a, 0x12eb1323, 0x1f05122e, 0x4f3f5317);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x02;    send[ 5] = 0x01;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x07;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0b003e00, 0x0000ff03, 0x2ddd0001, 0x30f7000b, 0x12eb1323, 0x1f05122e, 0x4f3f5317);
    process_table(ctx, 50, 0x01013f48, 0x02000200, 0x132d0001, 0x122e12eb, 0x53171f05, 0x00004f3f, 0x00000000);
    process_table(ctx, 50, 0x10003105, 0x9fff0032, 0x13350001, 0x122e12eb, 0x53171f05, 0x00004f3f, 0x00000000);
    process_table(ctx, 6, 0x17dd0100, 0x16be1886, 0x16a30000, 0x53171f0a, 0x00004f3f, 0x00000000, 0x00000000);
    process_table(ctx, 34, 0x17dd0000, 0x16ca1886, 0x1f0a16a3, 0x4f3f5317, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x0d0003c5, 0x0000ff03, 0x2ddd0001, 0x16e4000d, 0x1f0a16a8, 0x4f3f5317, 0x00000000);
}

void replay_f4(struct ctx *ctx)
{
    uint8_t send[64];
    uint8_t recv[64];

    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    fprintf(stderr, "// status ---------------------------------------\n");
    process_aux(ctx, 145, 1, 0, send, recv);


    process_table(ctx, 34, 0x00000100, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx,  6, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 76, 0x00840021, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 36, 0x3f480000, 0x0101000d, 0x10002100, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 35, 0x00000001, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 17, 0x00216978, 0x00002210, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 12, 0x3f480000, 0x0101000d, 0x10002100, 0x0007a120, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 49, 0x02300640, 0x003204b0, 0x00c00040, 0x00030001, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 35, 0x00000100, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 42, 0x00000300, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx,  4, 0x0f013f48, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx,  4, 0x10013f48, 0x00000011, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 50, 0x0f013f48, 0x02000200, 0x00000001, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 76, 0x01806978, 0x00000002, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx,  4, 0x08006978, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000);

    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    fprintf(stderr, "// status ---------------------------------------\n");
    process_aux(ctx, 145, 1, 0, send, recv);

    send[ 0] = 0x00;    send[ 1] = 0x06;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0x80;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x01;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x82;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x00;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x0a;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x07;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x10;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x09003e00, 0x0000ff03, 0x2ddd0001, 0x2fe70009, 0x12eb1323, 0x1f05122e, 0x4f3f5317);
    process_table(ctx, 76, 0x0b800000, 0x00010000, 0x320e0001, 0x13232ffd, 0x122e12eb, 0x53171f05, 0x00004f3f);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x01;    send[ 5] = 0x2e;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0a000000, 0x0000ff03, 0x2ddd0001, 0x3081010a, 0x12eb1323, 0x1f05122e, 0x4f3f5317);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x02;    send[ 5] = 0x01;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x03;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x83;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x0e;    send[ 1] = 0x00;    send[ 2] = 0x90;    send[ 3] = 0x40;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x02;    send[ 2] = 0x90;    send[ 3] = 0x45;
    send[ 4] = 0x00;    send[ 5] = 0x00;    send[ 6] = 0x00;    send[ 7] = 0x00;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    send[ 0] = 0x02;    send[ 1] = 0x01;    send[ 2] = 0x80;    send[ 3] = 0x50;
    send[ 4] = 0x00;    send[ 5] = 0x07;    send[ 6] = 0x00;    send[ 7] = 0x01;
    send[ 8] = 0x00;    send[ 9] = 0x00;    send[10] = 0x00;    send[11] = 0x00;
    send[12] = 0x00;    send[13] = 0x00;    send[14] = 0x00;    send[15] = 0x00;
    process_aux(ctx, 145, 1, 0, send, recv);
    process_table(ctx, 4, 0x0b003e00, 0x0000ff03, 0x2ddd0001, 0x30f7000b, 0x12eb1323, 0x1f05122e, 0x4f3f5317);
    process_table(ctx, 50, 0x01013f48, 0x02000200, 0x132d0001, 0x122e12eb, 0x53171f05, 0x00004f3f, 0x00000000);
    process_table(ctx, 50, 0x10003105, 0x9fff0032, 0x13350001, 0x122e12eb, 0x53171f05, 0x00004f3f, 0x00000000);
    process_table(ctx, 6, 0x17dd0100, 0x16be1886, 0x16a30000, 0x53171f0a, 0x00004f3f, 0x00000000, 0x00000000);
    process_table(ctx, 34, 0x17dd0000, 0x16ca1886, 0x1f0a16a3, 0x4f3f5317, 0x00000000, 0x00000000, 0x00000000);
    process_table(ctx, 4, 0x0d0003c5, 0x0000ff03, 0x2ddd0001, 0x16e4000d, 0x1f0a16a8, 0x4f3f5317, 0x00000000);
}

int main(int argc, char *argv[])
{
    struct ctx ctx;
    unsigned i;
    int r;

    ctx.bios_size = 64 << 10;
    ctx.bios = calloc(1, ctx.bios_size);
    if (ctx.bios == NULL) {
        return -ENOMEM;
    }
    if (radeon_pci_init(&ctx.pradeon)) {
        return -1;
    }
    r = pci_device_read_rom(ctx.pradeon.dev, ctx.bios);
    if (r) {
        goto out;
    }
    r = atombios_init(&ctx);
    if (r) {
        goto out;
    }

#if 0
#if 0
    for (i = 0x1724; i <= 0x1740; i+=4) {
        printf("radeon_mmio_wr32(&ctx.pradeon, 0x%08x, 0x%08x);\n", i, radeon_mmio_rd32(&ctx.pradeon, i));
    }
#else
radeon_mmio_wr32(&ctx.pradeon, 0x00001724, 0x00000002);
radeon_mmio_wr32(&ctx.pradeon, 0x00001728, 0x0008c000);
radeon_mmio_wr32(&ctx.pradeon, 0x0000172c, 0x00000000);
radeon_mmio_wr32(&ctx.pradeon, 0x00001730, 0x00000001);
radeon_mmio_wr32(&ctx.pradeon, 0x00001734, 0x00000000);
radeon_mmio_wr32(&ctx.pradeon, 0x00001738, 0x00000009);
radeon_mmio_wr32(&ctx.pradeon, 0x0000173c, 0x00001000);
radeon_mmio_wr32(&ctx.pradeon, 0x00001740, 0x00030108);
#endif

#else

#if 1
    fprintf(stderr, "ASIC INIT --------------------------\n");
    r = atom_asic_init(ctx.atom_ctx);
    if (r) {
        fprintf(stderr, "asic init failed\n");
        goto out;
    }
    fprintf(stderr, "ASIC REPLAY ------------------------\n");
    replay_f4(&ctx);
#else
    for (i = 0; i < (sizeof(logs)/sizeof(logs[0])); i++) {
        atom_trace_cmd_header(ctx.atom_ctx, logs[i].data[0], &logs[i].data[1]);
    }
#endif
#endif


out:
    atombios_fini(&ctx);
    radeon_pci_fini(&ctx.pradeon);
    return r;
}
